@@ -10,9 +10,10 @@ def set_parser(parser):
   parser.add_argument("-b", type=strlist, dest="ribobampaths", default=[], help="Ordinary riboseq bam files, comma seperated")
   parser.add_argument("-g", type=str, dest="genepath", required=True, help='Gene annotation file for ORF prediction')
   parser.add_argument("-f", type=str, dest="genomefapath", required=True, help="Genome fasta file")
-  parser.add_argument("-o", type=str, dest="output", required=True, help="Output result file")
+  parser.add_argument("-o", type=str, dest="output", default='predict.txt', required=True, help="Output result file")
   #### alt input options ####
   parser.add_argument("-i", type=str, dest="input", help="Only test input candidate ORFs, format: transID start stop (0 based, half open)")
+  parser.add_argument("--igenomepos", action="store_true", help="The start and end in -i input file are genome positions")
   parser.add_argument("--geneformat", type=str, default='auto', help="Gene annotation file format (gtf, bed, gpd, gff, default: auto)")
   parser.add_argument("--tispara", type=strlist, help="Input offset parameter files for -t bam files")
   parser.add_argument("--ribopara", type=strlist, help="Input offset parameter files for -b bam files")
@@ -22,6 +23,8 @@ def set_parser(parser):
   parser.add_argument("-s", type=str, dest="inestpath", help="Input background estimation result file instead of instant estimation")
   parser.add_argument("--transprofile", type=str, help="Output RPF P-site profile for each transcript")
   parser.add_argument("--inprofile", type=str, help="Input RPF P-site profile for each transcript, instead of reading bam reads, save time for re-running")
+  parser.add_argument("--chrmap", type=str, help="Input chromosome id mapping table file if annotation chr ids are not same as chr ids in bam/fasta files")
+  parser.add_argument("--allresult", type=str, help="All result output without FDR q-value threshold (default: output + '_all.txt', 'off' to turn off)")
   # predict options
   parser.add_argument("--alt", action="store_true", help="Use alternative start codons (all codons with 1 base different from ATG)")
   parser.add_argument("--altcodons", type=strlist, help="Use provided alternative start codons, comma seperated, eg. CTG,GTG,ACG")
@@ -34,14 +37,16 @@ def set_parser(parser):
   parser.add_argument("--genefilter", type=strlist, help="Only process given genes")
   parser.add_argument("--tpth", type=float, default=0.05, help="TIS p value threshold (default: 0.05)")
   parser.add_argument("--fpth", type=float, default=0.05, help="Frame p value threshold (default: 0.05)")
-  parser.add_argument("--minpth", type=float, default=0.05, help="At least one of TIS or frame p value should be lower than this threshold (default: 0.05)")
+  parser.add_argument("--minpth", type=float, default=1, help="At least one of TIS or frame p value should be lower than this threshold (default: 0.05)")
   parser.add_argument("--fspth", type=float, default=0.05, help="Fisher's p value threshold")
-  parser.add_argument("--fsqth", type=float, default=1, help="Fisher's FDR q value threshold")
+  parser.add_argument("--fsqth", type=float, default=0.05, help="Fisher's FDR q value threshold")
   parser.add_argument("--framelocalbest", action="store_true", help="Only report local best frame test results")
   parser.add_argument("--framebest", action="store_true", help="Only report best frame test results")
   parser.add_argument("--longest", action="store_true", help="Only report longest possible ORF results")
   parser.add_argument("--seq", action="store_true", help="Report ORF sequences")
   parser.add_argument("--aaseq", action="store_true", help="Report amino acid sequences")
+  parser.add_argument("--blocks", action="store_true", help="Report all exon block positions for predicted ORFs")
+  parser.add_argument("--inframecount", action="store_true", help="Report the sum of all counts at the in-frame positions in the ORF")
   # Reads filters
   parser.add_argument("--maxNH", type=int, default=5, help="Max NH value allowed for bam alignments (default: 5)")
   parser.add_argument("--minMapQ", type=float, default=1, help="Min MapQ value required for bam alignments (default: 1)")
@@ -51,13 +56,16 @@ def set_parser(parser):
   parser.add_argument("--compatiblemis", type=int, default=2, help="Missed bases allowed in reads compatibility check (default: 2)")
   # running options
   parser.add_argument("-p", type=int, dest="numProc", default=1, help="Number of processes")
-  parser.add_argument("-v", "--verbose", action="count", help="Increase output verbosity")
+  parser.add_argument("-v", "--verbose", action="count", default=0, help="Increase output verbosity")
 
 def strlist(s):
   '''Convert comma seperated file name string to list
   '''
   return s.split(',')
 
+try: imap = itertools.imap
+except: imap = map
+
 def run(args):
   '''Main function for ORF finding
   '''
@@ -65,8 +73,9 @@ def run(args):
   global tisbampaths, tisoffdict, ribobampaths, riboffdict, genomefapath, compatible, compatiblemis
   global minaalen, enrichtest, slp, paras, verbose, alt, title, tis2ribo, gfilter
   global tpth, fpth, minpth, fspth, framebest, framelocalbest, longest, transprofile, TIS_types #fspth
-  global paired, seq, aaseq # showtime
-  paired, seq, aaseq = args.paired, args.seq, args.aaseq
+  global paired, seq, aaseq, blocks, inframecount # showtime
+  paired, seq, aaseq, blocks = args.paired, args.seq, args.aaseq, args.blocks
+  inframecount = args.inframecount
   ribo.maxNH, ribo.minMapQ, ribo.secondary = args.maxNH, args.minMapQ, args.secondary
   tisbampaths = args.tisbampaths
   ribobampaths = args.ribobampaths
@@ -81,13 +90,22 @@ def run(args):
   transprofile = args.transprofile
   harrwidth = None
   TIS_types = ['Annotated', 'Truncated', 'Extended', "5'UTR", "3'UTR", 'Internal', 'Novel']
+  if args.chrmap is not None :
+    chrmap = {}
+    for lst in io.splitIter(args.chrmap, sep=None):
+      if len(lst) < 2: continue
+      chrmap[lst[0]] = lst[1]
+      chrmap[lst[1]] = lst[0]
+    bam.chrmap = chrmap
+    fa.chrmap = chrmap
   if args.harrwidth is not None : harrwidth = args.harrwidth
   elif args.harr : harrwidth = 15
   verbose = args.verbose
   alt = args.alt
   if args.altcodons is not None : 
     alt = True
-    orf.cstartlike = [c.upper() for c in args.altcodons]
+    if args.altcodons[0].upper() == 'ALL' : orf.cstartlike = orf.allcodons
+    else : orf.cstartlike = [c.upper() for c in args.altcodons]
   tpth, fpth, minpth, framebest, framelocalbest = args.tpth, args.fpth, args.minpth, args.framebest, args.framelocalbest # fspth
   fspth = args.fspth
   longest = args.longest
@@ -101,13 +119,13 @@ def run(args):
   tisoffdict = find_offset(args.tisbampaths, args.tispara)
   riboffdict = find_offset(args.ribobampaths, args.ribopara)
   if len(args.ribobampaths) == 0 : 
-    print('No ordinary riboseq data input. TIS data will also be used as ordinary riboseq data.')
+    print('No regular RiboSeq data input. TIS data will also be used as regular RiboSeq data.')
     tis2ribo = True
   if len(args.tisbampaths) == 1 : 
     if args.inestpath is None : 
       path = args.tisbampaths[0] + '.bgest.txt'
       if isfile(path) : args.inestpath = path
-      else : args.estpath = path
+      elif args.estpath is None or args.estpath == '' : args.estpath = path
   if args.agenepath is None : args.agenepath = args.genepath
 
   # load genome, fasta file indexing
@@ -119,8 +137,8 @@ def run(args):
     print('No input TIS data!')
     paras, slp = [(1,0.5)], [1] # No TIS input
   elif args.inestpath is None : #== '' :
-    print ("{} Estimate TIS parameters...".format(time.ctime()))
-    if args.verbose : print("TIS estimation result will be saved to {}".format(args.estpath))
+    print ("{} Estimating TIS background parameters...".format(time.ctime()))
+    if args.verbose : print("TIS background estimation result will be saved to {}".format(args.estpath))
     if args.numProc > 1 : 
       from multiprocessing import Process
       import multiprocessing.pool
@@ -135,16 +153,17 @@ def run(args):
         Process = NoDaemonProcess
 
       pool = MyPool(1) # This is for memory efficiency
-      paras, slp, data = pool.apply(ribo.estimateTISbg, args=(args.agenepath, args.tisbampaths, args.genomefapath), kwds={'parts': parts, 'offdict': tisoffdict, 'numProc': args.numProc, 'verbose': args.verbose, 'geneformat': args.geneformat, 'harrwidth': harrwidth, 'paired': paired})
+      paras, slp, data = pool.apply(ribo.estimateTISbg, args=(args.agenepath, args.tisbampaths, args.genomefapath), kwds={'parts': parts, 'offdict': tisoffdict, 'alt_tis': alt, 'numProc': args.numProc, 'verbose': args.verbose, 'geneformat': args.geneformat, 'harrwidth': harrwidth, 'paired': paired})
       pool.close()
     else : 
-      paras, slp, data = ribo.estimateTISbg(args.genepath, args.tisbampaths, args.genomefapath, parts = parts, offdict = tisoffdict, numProc = 1, verbose = verbose, geneformat = args.geneformat, harrwidth = harrwidth, paired = paired)
+      paras, slp, data = ribo.estimateTISbg(args.agenepath, args.tisbampaths, args.genomefapath, parts = parts, offdict = tisoffdict, alt_tis=alt, numProc = 1, verbose = verbose, geneformat = args.geneformat, harrwidth = harrwidth, paired = paired)
     estfile = open(args.estpath, 'w')
     for i in range(len(parts)):
       estfile.write("{}\t{}\t{}\t{}\t{}\n".format(paras[i][0], paras[i][1], parts[i], slp[i], data[i]))
     estfile.close()
     
   else : 
+    print("Loading TIS background file {}...".format(args.inestpath))
     inestfile = open(args.inestpath, 'r')
     paras, slp = [], []
     for l in inestfile:
@@ -152,6 +171,11 @@ def run(args):
       paras.append((float(lst[0]), float(lst[1])))
       slp.append(eval(lst[3]))
 
+  if args.inprofile is not None and not isfile(args.inprofile):
+    print('inprofile {} not found!'.format(args.inprofile))
+    if args.transprofile is None:
+      transprofile = args.inprofile
+
   if args.numProc > 1 : 
     from multiprocessing import Pool
     pool = Pool(processes = args.numProc - 1) 
@@ -160,16 +184,18 @@ def run(args):
   known_tis = {}
   if args.agenepath != args.genepath :
     if verbose : print('Loading CDS annotation...')
-    for g in io.geneIter(args.agenepath, fileType = args.geneformat, chrs = genome.idx, verbose = args.verbose):
+    for g in io.geneIter(args.agenepath, fileType = args.geneformat, verbose = args.verbose):
+      if genome.get_chrname(g.chr) is None: continue
       if g.chr not in cds_regions :
         cds_regions[g.chr] = {'+':[interval.Interval() for i in range(3)], '-':[interval.Interval() for i in range(3)]}
         known_tis[g.chr] = {'+':{}, '-':{}}
-      cr = interval.cds_region_gene(g)
-      for i in range(3) :
-        cds_regions[g.chr][g.strand][i].lst += cr[i].lst
-      for t in g.trans :
+      for t in g.trans:
+        cr = interval.cds_region_trans(t)
+        for i in range(3) :
+          cds_regions[t.chr][t.strand][i].lst += cr[i].lst
+      #for t in g.trans :
         tis = t.cds_start(cdna = False)
-        if tis is not None : known_tis[g.chr][g.strand][tis] = 1
+        if tis is not None : known_tis[t.chr][t.strand][tis] = 1
 
   inorf = None
   if args.input is not None :
@@ -184,21 +210,22 @@ def run(args):
       inorf[tid].append([tis, stop])
   inprofile = None
   if args.inprofile is not None :
-    if verbose : print('Loading transcript profile...')
-    inprofile = {}
-    for lst in io.splitIter(args.inprofile):
-      try : gid, tid, tispf, ribopf = lst[0], lst[1], eval(lst[3]), eval(lst[4])
-      except : continue
-      if gid not in inprofile : inprofile[gid] = {}
-      inprofile[gid][tid] = tispf, ribopf
+    if isfile(args.inprofile):
+      if verbose : print('Loading transcript profile...')
+      inprofile = {}
+      for lst in io.splitIter(args.inprofile):
+        try : gid, tid, tispf, ribopf = lst[0], lst[1], eval(lst[3]), eval(lst[4])
+        except : continue
+        if gid not in inprofile : inprofile[gid] = {}
+        inprofile[gid][tid] = tispf, ribopf
 
   print("{} Predicting...".format(time.ctime()))
   profile = exp.Profile()
   title = ['TISGroup', 'TISCounts', 'TISPvalue', 'RiboPvalue', 'RiboPStatus']
   j = [0,0] # total number of ORF/TIS for BH correction
-  gene_iter = io.geneIter(args.genepath, fileType = args.geneformat, chrs = genome.idx, verbose = args.verbose)
-  para_iter = genePara(gene_iter, inorf, inprofile)
-  if args.numProc <= 1 : pred_iter = itertools.imap(_pred_gene, para_iter)
+  gene_iter = io.geneIter(args.genepath, fileType = args.geneformat, verbose = args.verbose) # chrs = genome.idx
+  para_iter = genePara(gene_iter, inorf, inprofile, args.igenomepos)
+  if args.numProc <= 1 : pred_iter = imap(_pred_gene, para_iter)
   else : 
     #from multiprocessing import Pool
     #pool = Pool(processes = args.numProc - 1)
@@ -213,19 +240,20 @@ def run(args):
     j[1] += ji[1]
     for e in es : 
       profile.add_exp(e)
-      if verbose >= 2 : print e
+      if verbose >= 2 : print('{} {}'.format(time.ctime(), str(e)))
     if transprofile is not None : 
       for tid in tpfs :
         tpfile.write(io.tabjoin(tid, tpfs[tid])+'\n')
     if g.chr not in cds_regions : 
       cds_regions[g.chr] = {'+':[interval.Interval() for i in range(3)], '-':[interval.Interval() for i in range(3)]}
       known_tis[g.chr] = {'+':{}, '-':{}}
-    cr = interval.cds_region_gene(g)
-    for i in range(3) :
-      cds_regions[g.chr][g.strand][i].lst += cr[i].lst
-    for t in g.trans : 
+    for t in g.trans:
+      cr = interval.cds_region_trans(t)
+      for i in range(3) :
+        cds_regions[t.chr][t.strand][i].lst += cr[i].lst
+    #for t in g.trans : 
       tis = t.cds_start(cdna = False)
-      if tis is not None : known_tis[g.chr][g.strand][tis] = 1
+      if tis is not None : known_tis[t.chr][t.strand][tis] = 1
 
   for chr in cds_regions :
     for strand in cds_regions[chr] :
@@ -257,16 +285,32 @@ def run(args):
   s += '\tFisherPvalue\tTISQvalue\tFrameQvalue\tFisherQvalue\tAALen'
   if seq : s += '\tSeq'
   if aaseq : s += '\tAASeq'
+  if blocks: s += '\tBlocks'
+  if inframecount: s += '\tInFrameCount'
   s += '\n'
   outfile.write(s)
 
+  if args.allresult is not None and args.allresult.upper() == 'OFF' : allout = None
+  elif args.fsqth == 1: allout = None
+  else : 
+    if args.allresult is None : 
+      lst = args.output.split('.')
+      if lst[-1] == 'txt' : args.allresult = args.output[:-4] + '_all.txt' 
+      else : args.allresult = args.output + '_all.txt'
+    allout = open(args.allresult, 'w')
+    allout.write(s)
+
   for e in profile:
-    if e.q > args.fsqth : continue
+    #if e.q > args.fsqth : continue
+    if len(tisbampaths) == 0 : e.data[5], e.data[8] = None, None # No Fisher's
     s = "%s\t%d" % (e, e.length)
     if seq : s += '\t' + e.sq
     if aaseq : s += '\t' + e.aa
+    if blocks: s += '\t' + e.blocks
+    if inframecount: s += '\t{}'.format(e.inframecount)
     s += '\n'
-    outfile.write(s) # "%s\t%d\n" % (e, e.length)) #, e.sq))
+    if allout is not None : allout.write(s)
+    if e.q <= args.fsqth : outfile.write(s) # "%s\t%d\n" % (e, e.length)) #, e.sq))
   
 def check_overlap(e, known_tis, cds_regions) : 
   if e.tistype == 0 : return ''
@@ -276,7 +320,7 @@ def check_overlap(e, known_tis, cds_regions) :
       its = e.cr[i].intersect(cds_regions[i])
       if its.rlen() > 0 : return ':CDSFrameOverlap'
 
-def genePara(gene_iter, inorf, inprofile):
+def genePara(gene_iter, inorf, inprofile, igenomepos = False):
   '''Generate parameters (gene, candidates/None, profile) for function _pred_gene()
   '''
   if inorf is not None :
@@ -285,7 +329,19 @@ def genePara(gene_iter, inorf, inprofile):
       if inprofile is not None and g.id in inprofile : pf = inprofile[g.id]
       cand = {}
       for t in g.trans:
-        if t.id in inorf : cand[t.id] = inorf[t.id] #yield t, inorf[t.id]
+        if t.id not in inorf: continue
+        if not igenomepos: cand[t.id] = inorf[t.id] #yield t, inorf[t.id]
+        else:
+          c = []
+          for tis, stop in inorf[t.id]:
+            gtis, gstop = tis, stop
+            tis, stop = t.cdna_pos(tis), t.cdna_pos(stop)
+            if tis is None or stop is None:
+              print('Warning: skip input genome position not in transcript: {} {} {} {} {}'.format(t.id, gtis, gstop, tis, stop))
+              continue
+            if stop < tis: tis, stop = stop, tis
+            c.append([tis, stop])
+          cand[t.id] = c
       #if len(cand) > 0 : 
       yield g, cand, pf
   else :
@@ -320,6 +376,7 @@ def _pred_gene(ps): ### trans
   tpfs = {} #trans profiles
   genome = fa.Fa(genomefapath)
   has_tis = len(tisbampaths) > 0
+  if genome.get_chrname(g.chr) is None: return es, j, tpfs, g
 
   load = True
   if candidates is not None :
@@ -332,6 +389,9 @@ def _pred_gene(ps): ### trans
 
   for t in g.trans:
     #if candidates is not None and t.id not in candidates : continue
+    if t.strand not in ('+', '-'):
+      print('Warning: transcript {} {} {} strand "{}" unknown! '.format(t.gid, t.id, t.symbol, t.strand))
+      continue
     tl = t.cdna_length()
     if tl < ribo.minTransLen : continue # return es, j, tpfs, g ##
     #ttis = ribo.multiRibo(t, tisbampaths, offdict = tisoffdict, compatible = compatible)
@@ -351,8 +411,9 @@ def _pred_gene(ps): ### trans
       tribo = ribo.Ribo(t, bamload = ribombl, compatible = compatible, mis = compatiblemis)
 
     score = ttis.abdscore()
-    ip = ribo.pidx(score, slp)   
-    if verbose >= 2 : print g.id, t.id, ttis.total, tribo.total
+    if score is not None: ip = ribo.pidx(score, slp)   
+    else: ip = 0
+    if verbose >= 2 : print(io.tabjoin(g.id, t.id, ttis.total, tribo.total))
     cds1 = t.cds_start(cdna = True) 
     cds2 = t.cds_stop(cdna = True) 
     tsq = genome.transSeq(t)
@@ -379,9 +440,10 @@ def _pred_gene(ps): ### trans
         if tp is not None and tp < minp : minp = tp
         if minp > minpth : continue
         fsp, fss = stat.fisher_method([tp, rp]) #
-        if fsp > fspth : continue
+        if tp is not None and fsp > fspth : continue
         has_stop = tsq[stop-3:stop] in orf.cstop
         e = getResult(t, tis, stop, cds1, cds2, tsq, [ip, ttis.cnts[tis], tp, rp, 'N', fsp], has_stop)
+        if inframecount: e.inframecount = sum(tribo.cnts[tis:stop:3])
         es.append(e)
 
     else : #all possible ORFs
@@ -421,6 +483,7 @@ def _pred_gene(ps): ### trans
           fsp, fss = stat.fisher_method([tps[i], rps[i]]) #
           if fsp > fspth : continue
           e = getResult(t, tis, o.stop, cds1, cds2, tsq, [ip, ttis.cnts[tis], tps[i], rps[i], rst[i], fsp], o.has_stop_codon)
+          if inframecount: e.inframecount = sum(tribo.cnts[tis:o.stop:3])
           #tistype = tisType(tis, o.stop, cds1, cds2)
           #orfstr = '{}\t{}\t{}'.format(tsq[tis:tis+3],tis,o.stop)
           #tid = "%s\t%s\t%s\t%s\t%s:%d-%d:%s\t%s\t%s" % (t.gid, t.id, t.symbol, t.genetype, t.chr, t.genome_pos(tis), t.genome_pos(o.stop), t.strand, orfstr, tistype)
@@ -441,15 +504,18 @@ def _pred_gene(ps): ### trans
 def getResult(t, tis, stop, cds1, cds2, tsq, values, has_stop = True):
   tistype = tisType(tis, stop, cds1, cds2)
   orfstr = '{}\t{}\t{}'.format(tsq[tis:tis+3], tis, stop)
-  gtis, gstop = t.genome_pos(tis), t.genome_pos(stop)
+  gtis, gstop = t.genome_pos(tis, bias=1), t.genome_pos(stop, bias=0)
   if t.strand != '-' : genomestr = '{}:{}-{}:{}'.format(t.chr, gtis, gstop, t.strand)
   else : genomestr = '{}:{}-{}:{}'.format(t.chr, gstop, gtis, t.strand)
   tid = "%s\t%s\t%s\t%s\t%s\t%s\t%s" % (t.gid, t.id, t.symbol, t.genetype, genomestr, orfstr, TIS_types[tistype])
   e = exp.Exp(tid, values)
   e.length = (stop - tis) / 3 - 1
   if not has_stop : e.length += 1
-  if seq : e.sq = tsq[tis:stop]
+  if seq or aaseq : e.sq = tsq[tis:stop]
   if aaseq : e.aa = orf.translate(e.sq)
+  if blocks: 
+    oi = interval.trans2interval(t, tis, stop)
+    e.blocks = ','.join(['{}-{}'.format(r[0],r[1]) for r in oi.lst]) 
   e.chr, e.strand, e.tistype = t.chr, t.strand, tistype
   e.gtis, e.gstop = gtis, gstop
   if tistype == 2 : e.cr = interval.cds_region_trans(t, tis, tis+3) # Extended TIS region
@@ -457,7 +523,7 @@ def getResult(t, tis, stop, cds1, cds2, tsq, values, has_stop = True):
   return e
 
 def pvalStatus(ps) : 
-  '''Find global best (T) & local best (L) TISs that correspond to same stop codon, suggesting potential TISs by ordinary riboseq data
+  '''Find global best (T) & local best (L) TISs that correspond to same stop codon, suggesting potential TISs by regular RiboSeq data
   '''
   l = len(ps)
   st = ['N'] * l
