@@ -14,15 +14,34 @@ def changechr(chr):
   elif chr[0:3] == 'chr' : return chr[3:]
   else : return chr
 
+chrmap = {}
+    
 class Bamfile(pysam.Samfile):
   def __repr__(self):
     return 'pysam.Samfile '+self.filename
+  def get_chrname(self, chr) : #, chrmap = chrmap):
+    #sprint(chrmap)
+    if chr in self.references : return chr
+    if chr in chrmap :
+      chr1 = chrmap[chr]
+      if chr1 in self.references : return chr1
+      else : 
+        chr2 = changechr(chr1)
+        if chr2 in self.references : return chr2
+    chr1 = changechr(chr)
+    if chr1 == chr : return None
+    if chr1 in self.references : return chr1
+    elif chr1 in chrmap : 
+      chr2 = chrmap[chr1]
+      if chr2 in self.references : return chr2
+    return None
   def fetch_reads(self, chr, start, stop, maxNH = None, minMapQ = None, secondary = False, paired = False) : #, multiple_iterators=False):
-    if chr not in self.references : 
-      chr = changechr(chr)
-      if chr not in self.references : 
-        print("chr {} not found in Bamfile!".format(chr))
-        raise StopIteration
+    chr0, chr = chr, self.get_chrname(chr)
+    #if chr not in self.references : 
+      #chr = changechr(chr)
+    if chr is None : 
+      print("chr {} not found in Bamfile!".format(chr0))
+      return #raise StopIteration
     rds = self.fetch(reference=chr, start=start, end=stop) #, multiple_iterators=multiple_iterators)
     r1, r2 = {}, {}
     for read in rds:
@@ -133,7 +152,9 @@ class Bam():#AlignedRead
     else : return "Bam paired AlignedSegment {} {}".format(self.id, str(self))
   def __str__(self):
     return "%s:%d-%d:%s" % (self.chr, self.fragment_start, self.fragment_stop, self.strand)
-  
+
+  def get_tag(self, k):
+    return self.read.get_tag(k)
   def cdna_length(self): 
     return self.read.query_alignment_length
     #else : return self.fragment_length()
@@ -239,7 +260,7 @@ class Bam():#AlignedRead
         else: p1 -= l
       elif ctype in [2,3]: # D: deletion from the reference, N: skipped region from the reference
         pos += l
-    return None
+    return pos
   @property
   def introns(self):
     from . import bed
@@ -403,14 +424,39 @@ class Bam():#AlignedRead
       if self.read2.get_tag('MD')[-1] == '0' : 
         if not self.read2.get_tag('MD')[-2].isdigit() : return True
     return False
-def compatible_bam_iter(bamfile, trans, mis = 0, sense = True, maxNH = None, minMapQ = None, secondary = False): 
+  def mismatches(self):
+    import re
+    mm = {}
+    md = self.read.get_tag('MD')
+    ns = re.split('[ATGC^]+', md)
+    ms = re.split('\d+', md)
+    l = self.cdna_length()
+    p = 0
+    for i, n in enumerate(ns):
+      n = int(n)
+      if ms[i] != '':
+        q = p
+        p += len(ms[i])
+        if ms[i].startswith('^'): p -= 1
+        if self.is_reverse(): q = l - p
+        mm[q] = ms[i]
+      p += n
+    return mm
+
+
+def compatible_bam_iter(bamfile, trans, mis = 0, sense = True, maxNH = None, minMapQ = None, secondary = False, flank = 0): 
   '''compatible version of transReadsIter, slightly different
   '''
-  chr = trans.chr
-  if chr not in bamfile.references : 
-      chr = changechr(chr)
-      if chr not in bamfile.references : raise StopIteration
-  rds = bamfile.fetch(reference=chr, start=trans.start, end=trans.stop)#, multiple_iterators=False)
+  chr = bamfile.get_chrname(trans.chr)
+  if chr is None : return #raise StopIteration
+    #print("chr {} not found in Bamfile!".format(chr0))
+    #raise StopIteration
+  #chr = trans.chr
+  #if chr not in bamfile.references : 
+      #chr = changechr(chr)
+      #if chr not in bamfile.references : raise StopIteration
+  if flank < 0: flank = 0
+  rds = bamfile.fetch(reference=chr, start=trans.start-flank, end=trans.stop+flank)#, multiple_iterators=False)
   #introns = trans.introns
   for r in rds:
     read = Bam(r, bamfile)
@@ -422,27 +468,33 @@ def compatible_bam_iter(bamfile, trans, mis = 0, sense = True, maxNH = None, min
     except: pass
     if not secondary and r.is_secondary : continue
     if minMapQ is not None and read.read.mapping_quality < minMapQ : continue
-    o = read.read.get_overlap(trans.start, trans.stop)
+    o = read.read.get_overlap(trans.start-flank, trans.stop+flank)
     if o < read.cdna_length() - mis: 
       continue
     if read.is_compatible(trans = trans, mis = mis) :
       yield read
     #else:
       #print read.id, b, read.cdna_length()
-def transReadsIter(bamfile, trans, compatible = True, mis = 0, sense = True, maxNH = None, minMapQ = None, secondary = False, paired = False):
+def transReadsIter(bamfile, trans, compatible = True, mis = 0, sense = True, maxNH = None, minMapQ = None, secondary = False, paired = False, flank = 0):
   '''fetch reads from transcript exons
   '''
-  chr = trans.chr
-  if chr not in bamfile.references : 
-      chr = changechr(chr)
-      if chr not in bamfile.references : raise StopIteration
+  chr = bamfile.get_chrname(trans.chr)
+  if chr is None : 
+    #print("chr {} not found in Bamfile!".format(trans.chr))
+    return #raise StopIteration
   if compatible : 
     from . import interval
     transitv = interval.trans2interval(trans)
+  if flank < 0: flank = 0
   used = {}
-  #trans.exons = trans.exons ##
   for e in trans.exons : 
-    rds = bamfile.fetch_reads(chr=chr, start=e.start, stop=e.stop, maxNH = maxNH, minMapQ = minMapQ, secondary = secondary, paired = paired)#, multiple_iterators=False)
+    start, stop = e.start, e.stop
+    if flank > 0:
+      if start == trans.start:
+        start -= flank
+        if start < 1: start = 1
+      if stop == trans.stop: stop += flank
+    rds = bamfile.fetch_reads(chr=chr, start=start, stop=stop, maxNH = maxNH, minMapQ = minMapQ, secondary = secondary, paired = paired)#, multiple_iterators=False)
     for read in rds: #yield read
       #read = Bam(r, bamfile)
       if (read.id, read.fragment_start) in used : continue
@@ -529,17 +581,21 @@ class BamLoadChr:
     else : self.data = {strand:{}}
     if bampath is None : return
     bamfile = Bamfile(bampath)
-    if chr not in bamfile.references : 
-      chr = changechr(chr)
-      if chr not in bamfile.references : 
-        print("chr {} not found in Bamfile!".format(self.chr))
-        return
+    chr = bamfile.get_chrname(chr)
+    if chr is None : #raise StopIteration
+    #if chr not in bamfile.references : 
+      #chr = changechr(chr)
+      #if chr not in bamfile.references : 
+      print("chr {} not found in Bamfile!".format(self.chr))
+      return
+    self.chr = chr
     if region is None : region = [[None, None]]
     for start, stop in region : 
       for r in bamfile.fetch_reads(chr, start, stop, maxNH = maxNH, minMapQ = minMapQ, secondary = secondary, paired = paired):
         if r.strand not in self.data : continue
         p = posFunc(r)
         #print p
+        if p is None: continue
         if start is not None and p < start : continue
         if stop is not None and p > stop : continue
         if p not in self.data[r.strand] : self.data[r.strand][p] = {}
@@ -553,7 +609,7 @@ class BamLoadChr:
   def transCounts(self, trans, compatible = False, mis = 2):
     if self.strand != '.' and self.strand != trans.strand :
       print('Strand not match: {}, {}'.format(self, trans.id))
-      return
+      return []
     d = self.data[trans.strand]
     if compatible : 
       from . import interval
