@@ -2,6 +2,10 @@
 Bed format annotation processing
 Copyright (c) 2016 Peng Zhang <zhpn1024@163.com>
 '''
+def cmp3(a, b):
+  ''' cmp for python3'''
+  return (a > b) - (a < b)
+
 class Bed3:
   ''' Bed3 class for bed3 annotation format, father of Bed6
   '''
@@ -15,15 +19,16 @@ class Bed3:
     if type(x)==str:
       l = x.strip().split('\t')
       if bin: l[0:1] = []
-    elif type(x) == type(self) : l = [getattr(x, i) for i in self.Header]
+    elif type(x) == list or type(x) == tuple: l = x
+    #elif type(x) == type(self) : l = [getattr(x, i) for i in self.Header]
     elif type(x)==dict: l=[x[i] for i in self.Header]
-    else: l = x
+    else: l = [getattr(x, i) for i in self.Header]
     lst=[]
     for i in range(self.n):
       try: lst.append(self.Format[i](l[i]))
       except: 
         if i in (6, 7) : lst.append(lst[1]) # blockstart/stop
-        elif i in (8,) : lst.append(0) # rgb
+        elif i in (8,) : lst.append('0') # rgb
         elif i == 9 : lst.append(1)
         elif i == 10 : lst.append((lst[2]-lst[1],))
         elif i == 11 : lst.append((0,))
@@ -53,18 +58,18 @@ class Bed3:
   @property
   def id(self): #Bed3 only
     return "noname"
-  @property
-  def tid(self):
-    return self.id
-  @property
-  def gid(self):
-    return self.tid
-  @property
-  def symbol(self):
-    return self.gid
-  @property
-  def genetype(self):
-    return ""
+  #@property
+  #def tid(self):
+    #return self.id
+  #@property
+  #def gid(self):
+    #return self.tid
+  #@property
+  #def symbol(self):
+    #return self.gid
+  #@property
+  #def genetype(self):
+    #return ""
   @property
   def score(self): #Bed3 only
     return 0.0
@@ -101,6 +106,14 @@ class Bed3:
     return self.items[i]
   def __cmp__(self, other): #All bed
     return cmp(self.chr,other.chr) or cmp(self.start,other.start) or cmp(self.stop,other.stop)
+  def cmp(self, other):
+    return cmp3(self.chr,other.chr) or cmp3(self.start,other.start) or cmp3(self.stop,other.stop)
+  def __eq__(self, other):
+    return self.cmp(other) == 0
+  def __lt__(self, other):
+    return self.cmp(other) < 0
+  def __gt__(self, other):
+    return self.cmp(other) > 0
   def __sub__(self, other):
     return sub(self, other)
   def intersect(self, other):
@@ -187,8 +200,20 @@ class Bed3:
     return False
   def is_sense(self,other): #In same strand? All bed
     return self.strand==other.strand
-  def cdna_pos(self, p, strict = False): #Bed3 and Bed6
+
+  def flank_pos(self, p, flank = 100):
+    if flank < 0: return None
+    if self.start - flank <= p <= self.start:
+      if self.strand != '-': return p - self.start # upstream
+      else: return self.start - p + self.cdna_length()
+    if self.stop <= p <= self.stop + flank:
+      if self.strand != '-': return p - self.stop + self.cdna_length()
+      else: return self.stop - p # upstream
+    return None
+
+  def cdna_pos(self, p, strict = False, flank = 0): #Bed3 and Bed6
     if self.is_contain(p, strict): return abs(p-self.end5)
+    elif flank > 0: return self.flank_pos(p, flank)
     else: return None
   def genome_pos(self, p, bias = 1): #Bed3 and Bed6
     m = self.cdna_length()
@@ -278,6 +303,7 @@ class Bed6(Bed3):
 
 def com2tup(s): #comma string to tuple
   if type(s)==tuple: return s
+  if type(s)==list: return tuple(s)
   return tuple(map(int,s.strip().strip(',').split(',')))
 def tup2com(t): #tuple to comma string
   if type(t)==str: return t
@@ -392,13 +418,21 @@ class Bed12(Bed6):
     if self.strand=="-": self._introns = a[::-1]
     else : self._introns = a
     return self._introns
-  
-  def cdna_pos(self, p, strict = False):
+
+  def abs2relative(self):
+    '''change absolute blockStart values to relative values
+    '''
+    blockstarts = [st - self.start for st in self.blockStarts]
+    return self(blockStarts = blockstarts)
+
+  def cdna_pos(self, p, strict = False, flank = 0):
     '''if strict is True, the 3' end of exon will be considered as not in the transcript,
     if strict is False, 3' end of exon will be considered as start of the next exon, 
     or transcript end (self.cdna_length()) if in the last exon.
     '''
-    if not self.is_contain(p, strict) : return None
+    if not self.is_contain(p, strict) : # return None
+      if flank <= 0: return None
+      else: return self.flank_pos(p, flank)
     p1 = p - self.start # genome relative posistion
     pos = 0 # cDNA pos
     l=list(range(self.blockCount))
@@ -412,6 +446,7 @@ class Bed12(Bed6):
         else: pos+=p1-self.blockStarts[i]
         return pos
       else: pos+=self.blockSizes[i]
+      #print(pos)
     else: return None
 
   def genome_pos(self, p, bias=1):
@@ -517,7 +552,8 @@ def bed12_fetch(file, id, bin = False, verbose = False):
 
 class refGene(Bed12): # start with bins
   def __init__(self, x):
-    l = x.strip().split('\t')
+    if type(x) == list: l = x
+    else: l = x.strip().split('\t')
     self.exonStarts = com2tup(l[9])
     self.exonEnds = com2tup(l[10])
     start = int(l[4])
@@ -569,13 +605,17 @@ class gpdGene:
   def __init__(self, id = '', b = None):
     if b is None :
       self.trans = []
-      self.chr, self.start, self.stop, self.strand = '', None, None, '.'
+      self.chr, self.start, self.stop, self.strand = '', None, None, ''
       self.id = id
     else : 
       self.trans = [b]
       self.chr, self.start, self.stop, self.strand = b.chr, b.start, b.stop, b.strand
       self.id = b.gid
   def add_trans(self, tr):
+    if self.strand in ('+', '-') and tr.strand != self.strand :
+      print('Inconsistent trans strand: {} {} {} {}'.format(tr.gid, tr.tid, tr.strand, self.strand))
+      self.strand = '.'
+    elif self.strand == '': self.strand = tr.strand
     self.trans.append(tr)
   def __repr__(self):
     s = "gene_id " + self.id + ', ' + str(len(self.trans)) + " transcripts, " + Exon.__repr__(self)
@@ -585,7 +625,7 @@ class gpdGene:
   def check(self):
     if len(self.trans) > 0 :
       if self.chr == '' : self.chr = self.trans[0].chr
-      if self.trand == '.' : self.strand = self.trans[0].strand
+      if self.trand == '' : self.strand = self.trans[0].strand
       if self.start is None : self.start, self.stop = self.trans[0].start, self.trans[0].stop
     for t in self.trans:
       #t.check()
