@@ -4,13 +4,27 @@ Copyright (c) 2016 Peng Zhang <zhpn1024@163.com>
 '''
 
 def add_chr(chr):
-  if chr.isdigit() or chr in ('X','Y','M',): chr = 'chr' + chr
-  elif chr == 'MT' : chr = 'chrM' 
+  if len(chr) <= 3 and (chr.isdigit() or chr in ('X','Y',)): chr = 'chr' + chr
+  elif chr in ('M','MT','MtDNA','mitochondrion_genome','Mito') : chr = 'chrM' 
+  elif len(chr) == 2 and chr[0].isdigit(): chr = 'chr' + chr
+  elif chr in ('I','II','III','IV','V','VI','VII','VIII','IX','XI','XII','XIII','XIV','XV','XVI'): chr = 'chr' + chr
   return chr
 def rm_chr(chr):
   if chr[3:].isdigit() or chr in ('chrX','chrY'): chr = chr[3:]
   elif chr == 'chrM' : chr = 'MT'
   return chr
+def changechr(chr):
+  '''change between two chr versions
+  '''
+  if chr.isdigit() or chr in ('X','Y','M'): return 'chr' + chr
+  elif chr == 'MT' : return 'chrM'
+  elif chr == 'chrM' : return 'MT'
+  elif chr[0:3] == 'chr' : return chr[3:]
+  else : return chr
+def cmp3(a, b):
+  ''' cmp for python3'''
+  return (a > b) - (a < b)
+
 class Exon:
   '''exon in gtf format, 0 based, similar as bed
   '''
@@ -22,6 +36,7 @@ class Exon:
     self.type, self.score, self.attrstr = lst[2], lst[5], lst[8]
     self.gff, self.addchr, self.frame = gff, addchr, lst[7]
     self.lst = lst
+    self.exons = [self]
   def __repr__(self):
     return self.chr + ':'+str(self.start)+'-'+str(self.stop)+':'+self.strand
   def __str__(self, sep = '\t', new = False):
@@ -34,6 +49,14 @@ class Exon:
     return "%s:%d-%d:%s" % (self.chr, self.start, self.stop, self.strand)
   def __cmp__(self, other):
     return cmp(self.chr,other.chr) or cmp(self.start,other.start) or cmp(self.stop,other.stop)
+  def cmp(self, other):
+    return cmp3(self.chr,other.chr) or cmp3(self.start,other.start) or cmp3(self.stop,other.stop)
+  def __eq__(self, other):
+    return self.cmp(other) == 0
+  def __lt__(self, other):
+    return self.cmp(other) < 0
+  def __gt__(self, other):
+    return self.cmp(other) > 0
   def __len__(self): 
     return self.stop - self.start
   def attr(self, key):
@@ -41,18 +64,22 @@ class Exon:
     '''
     s = self.attrstr
     if self.gff : 
-      p1 = s.find(key)
+      p1 = s.find(key+'=')
+      if p1 < 0 : p1 = s.find(key+':')
       if p1 < 0 : return ''
-      else : p1 += len(key)
+      else : p1 += len(key) #+ 1
       p2 = s.find(';', p1)
-      if s[p1] == '=' : return s[p1:p2]
-      elif s[p1] == ':' : return s[p1:p2].split(',')[0]
+      if p2 == -1 : p2 = None
+      if s[p1] == '=' : return s[p1+1:p2]
+      elif s[p1] == ':' : return s[p1+1:p2].split(',')[0]
       else : return ''
     p1 = s.find(key + ' ')
     if p1 < 0 : return ''
     else : p1 += len(key) + 1
     p2 = s.find(';', p1)
-    return eval(s[p1:p2])
+    if p2 == -1 : p2 = None
+    try: return eval(s[p1:p2])
+    except: return s[p1:p2]
   @property
   def length(self):
     return len(self)
@@ -69,7 +96,7 @@ class Exon:
   def gid(self):
     try : return self.gid_c # gid cache
     except : 
-      if self.gff : self.gid_c = self.attr('GeneID')
+      if self.gff and self.attr('GeneID') != '' : self.gid_c = self.attr('GeneID')
       else : self.gid_c = self.attr('gene_id')
     return self.gid_c
   @property
@@ -77,16 +104,33 @@ class Exon:
     try : return self.tid_c
     except : 
       self.tid_c = self.attr('transcript_id')
+      if self.gff: # and self.tid_c == '':
+        if self.lst[2] in ('transcript', 'mRNA', 'tRNA', 'rRNA'):
+          self.tid_c = self.id
+        else:
+          p = self.attr('Parent')
+          if p.startswith('rna-'): self.tid_c = p[4:]
+          elif p.startswith('gene-'): self.tid_c = p[5:]
+          elif p.startswith('rna'): self.tid_c = p
+          elif p.startswith('gene'): self.tid_c = p
       return self.tid_c
   @property
   def symbol(self):
     try : return self.sym_c
     except : 
       self.sym_c = self.attr('gene_name')
+      if self.gff and self.sym_c == '': self.sym_c = self.attr('gene')
     return self.sym_c
   @property
   def id(self):
-    return self.attr('exon_id')
+    try: return self._id
+    except:
+      self._id = self.attr('exon_id')
+      if self._id == '': self._id = self.attr('ID')
+      return self._id
+  @id.setter
+  def id(self, value):
+    self._id = value
   @property
   def end5(self): #5' end, all bed
     if self.strand != '-' : return self.start
@@ -138,8 +182,19 @@ class Exon:
     else : 
       if strict : return p <= self.start
       else : return p < self.start
-  def cdna_pos(self, p, strict = False): # exon only
+  def flank_pos(self, p, flank = 100):
+    if flank < 0: return None
+    if self.start - flank <= p <= self.start:
+      if self.strand != '-': return p - self.start # upstream
+      else: return self.start - p + self.cdna_length()
+    if self.stop <= p <= self.stop + flank:
+      if self.strand != '-': return p - self.stop + self.cdna_length()
+      else: return self.stop - p # upstream
+    return None
+
+  def cdna_pos(self, p, strict = False, flank = 0): # exon only
     if self.is_contain(p, strict): return abs(p-self.end5)
+    elif flank > 0: return self.flank_pos(p, flank)
     else: return None
   def genome_pos(self, p, bias = 1): # exon only
     m = self.cdna_length()
@@ -156,9 +211,9 @@ class Exon:
     #self.attr = other.attr
     self.gff, self.addchr, self.frame = other.gff, other.addchr, other.frame
     self.lst = other.lst
-  @property
-  def exons(self):
-    return [self]
+  #@property
+  #def exons(self):
+    #return [self]
   def cds_start(self, cdna = False): return None
   def cds_stop(self, cdna = False): return None
     
@@ -203,7 +258,8 @@ class gtfTrans(Exon):
   '''
   def __init__(self, lst, gff = False, addchr = False): 
     Exon.__init__(self, lst, gff, addchr)
-    self.id = self.tid
+    if self.tid != '': self.id = self.tid
+    elif self.id != '': self.tid_c = self.id
     #self.type = lst[1]
     self.exons = []
     self.cds = []
@@ -269,7 +325,9 @@ class gtfTrans(Exon):
     exonStops.sort()
     lst += [tup2com(exonStarts), tup2com(exonStops)]
     return '\t'.join(map(str, lst))
-  def check(self):
+  def check(self, final = True):
+    if final and len(self.exons) == 0:
+      if len(self.cds) > 0: self.exons += self.cds
     for e in self.exons:
       if e.start < self.start: self.start = e.start
       if e.stop > self.stop: self.stop = e.stop
@@ -333,12 +391,11 @@ class gtfTrans(Exon):
     sc = self.stop_codon # Determine by stop codon
     if len(sc) != 3 : 
       for e in self.exons:
-        if sc.end3 == e.end3 : 
+        if sc.end5 == e.end5 : break
+        elif sc.end3 == e.end3 : # bug elif 
           cs = self.cdna_pos(sc.end5) + 3
           if cdna : return cs
           else : return self.genome_pos(cs, 0)
-        if sc.end5 == e.end5 :
-          break
       else : 
         print('Stop codon error : %s %s %s' % (self.gid, self.id, sc.short_str()))
     gs = sc.end3
@@ -395,22 +452,20 @@ class gtfTrans(Exon):
       last = e.end3
     self._introns = introns
     return introns
-  def cdna_pos(self, p, strict = False):
+  def cdna_pos(self, p, strict = False, flank = 0):
     '''if strict is True, the 3' end of exon will be considered as not in the transcript,
     if strict is False, 3' end of exon will be considered as start of the next exon, 
     or transcript end (self.cdna_length()) if in the last exon.
+    if flank is not 0, TSS and TTS flanking regions are returned with negative distance and
+    cDNA length + distance
     '''
-    if not self.is_contain(p, strict) : return None
-    #if p < self.start or p > self.stop : return None
+    if not self.is_contain(p, strict) :
+      if flank <= 0: return None
+      else: return self.flank_pos(p, flank)
     pos = 0
     for e in self.exons:
       if e.is_upstream(p, strict) : return None
       if e.is_contain(p, strict) : return pos + abs(p - e.end5) # e.start <= p <= e.stop:
-        #if self.is_reverse():
-          #pos += e.stop - p
-        #else:
-          #pos += p - e.start
-        #return pos
       pos += len(e)
     return None
   def is_exon(self, p, strict = False):
@@ -451,9 +506,10 @@ class gtfGene(Exon):
     self.trans = []
     #self.type = lst[1]
   def add_trans(self, tr):
-    if tr.strand != self.strand :
-      print('Wrong trans strand: {} {} {} {}'.format(tr.gid, tr.tid, tr.strand, self.strand))
-      tr.strand = self.strand
+    if self.strand in ('+', '-') and tr.strand != self.strand :
+      print('Inconsistent trans strand: {} {} {} {}'.format(tr.gid, tr.tid, tr.strand, self.strand))
+      #tr.strand = self.strand
+      self.strand = '.'
     self.trans.append(tr)
     #self.check()
   def __repr__(self):
@@ -595,7 +651,9 @@ def gtfgene_iter(fin, filt = [], gff = False, addchr = False, chrs = None, verbo
     if len(lst) < 2 : continue
     if lst[2] == 'region' : continue
     if lst[0] != chr:
-      if chrs is not None and lst[0] not in chrs : continue
+      if chrs is not None and lst[0] not in chrs and changechr(lst[0]) not in chrs: 
+        #print('{}\t{}\t{}'.format(lst[0], changechr(lst[0]), chrs))
+        continue
       if verbose : print(lst[0])
       for gid in gidlist:
         if gid in genes : 
@@ -623,7 +681,7 @@ def gtfgene_iter(fin, filt = [], gff = False, addchr = False, chrs = None, verbo
       genes[g.id] = g
       genes2[g.id] = g
       #if g.id not in gidlist: gidlist.append(g.id)
-    elif lst[2] == 'transcript':
+    elif lst[2] in ('transcript', 'mRNA', 'tRNA', 'rRNA'):
       t = gtfTrans(lst, gff, addchr)
       if t.gid not in genes:
         g = gtfGene(lst, gff, addchr)
@@ -677,7 +735,7 @@ def gtftrans_iter(fin, filt = [], gff = False, addchr = False, chrs = None, verb
     if len(lst) < 2 : continue
     if lst[2] == 'region' : continue
     if lst[0] != chr:
-      if chrs is not None and lst[0] not in chrs : continue
+      if chrs is not None and lst[0] not in chrs and changechr(lst[0]) not in chrs: continue
       if verbose : print(lst[0])
       for tid in tidlist:
         if tid in trans : 
