@@ -3,7 +3,7 @@ Main library for riboseq analysis in zbio/ribotish
 Copyright (c) 2016 Peng Zhang <zhpn1024@163.com>
 '''
 import itertools, math
-import stat, bam, gtf, exp, orf, tools, io, fa, interval
+from . import stat, bam, gtf, exp, orf, tools, io, fa, interval
 from multiprocessing import Pool
 
 codonSize = 3
@@ -16,6 +16,11 @@ defOffset = 12
 #nhead, ntail = defOffset, 30 - defOffset
 nhead = ntail = 0
 
+try: imap = itertools.imap
+except: imap = map
+try: izip = itertools.izip
+except: izip = zip
+
 def offset(read = 30, offdict = None):
   '''get P-site offset by read length, default: 12
   '''
@@ -47,7 +52,7 @@ def rstest(x, y, delta = 1e-4, show = False):
 class Ribo: 
   '''riboseq profile for a transcript
   '''
-  def __init__(self, trans, ribobam = None, bamload = None, offset = offset, offdict = None, maxNH = maxNH, minMapQ = minMapQ, secondary = secondary, compatible = True, mis = 2, downsample = 1.0, seed = 1, saverid = False, paired = False):
+  def __init__(self, trans, ribobam = None, bamload = None, offset = offset, offdict = None, compatible = True, mis = 2, downsample = 1.0, seed = 1, saverid = False, paired = False):
     self.length = trans.cdna_length()
     self.nhead, self.ntail = nhead, ntail
     self.trans = trans
@@ -135,6 +140,7 @@ class Ribo:
       #print i
       if i % codonSize == frame : inarr.append(self.cnts[i])
       else : outarr.append(self.cnts[i])
+    #print(self.trans.id, len(inarr), len(outarr))
     if len(inarr) <= 0 or len(outarr) <= 0 : return None
     if max(inarr) == 0 : return 1 ## 
     if old : return rstest_mw(inarr, outarr)
@@ -155,14 +161,14 @@ class Ribo:
     if glm : return stat.glmNBTest(inarr, outarr)
     p = rstest(inarr, outarr, delta = delta)
     if show : 
-      print 'inarr', inarr
-      print 'outarr', outarr
+      print('inarr: {}'.format(inarr))
+      print('outarr: {}'.format(outarr))
     return p
   
   def multi_orf_test(self, orflist, glm = False): 
     '''multiple ORF detection for riboseq data, not used in final version
     '''
-    import interval
+    from . import interval
     tid = self.trans.id
     blank = []
     for i in range(codonSize): ## Blank regions in different frames
@@ -214,12 +220,13 @@ class Ribo:
   def efpvalues(self, orf, blank, glm = False, show = False): # For multi_orf_test
     if orf.indr.rlen() < minRlen : r1 = orf.region
     else : r1 = orf.indr
+    #print(self.trans.id, r1)
     fp = self.frame_test_region(r1, orf.frame(), glm = glm)
     r2 = blank[codonSize]
     if r2.rlen() < minRlen or orf.indr.rlen() < minRlen : r2 = blank[orf.frame()]
     ep = self.enrich_test_region(r1, r2, orf.frame(), glm = glm, show = show)
     if show : 
-      print r1, r2, fp, ep, orf.frame()
+      print(r1, r2, fp, ep, orf.frame())
       #print self.cnts[0:1000]
       #print self.cnts[1000:2000]
     ''' More complicated tests
@@ -241,7 +248,7 @@ class Ribo:
   def max_orf(self, orf, blank): 
     '''for ribonly_orf_finder
     '''
-    import interval
+    from . import interval
     indr = indep_region(orf, blank)
     mst, mp = None, 1
     for st in orf.sts :
@@ -256,7 +263,7 @@ class Ribo:
     '''find likely TIS by only riboseq data, not used, similar function in pvalStatus()
     However, this function seems better
     '''
-    import interval
+    from . import interval
     blank = []
     for i in range(codonSize): ## Blank regions in different frames
       b = interval.Interval(start=self.nhead, stop=self.length-self.ntail)
@@ -427,8 +434,8 @@ def estimate_tis_bg_all(gtfpath, bampath, genomefapath, parts = [0.25, 0.5, 0.75
   data = [exp.ReadDict() for i in range(len(parts))]
   
   gene_iter = gtf.gtfGene_iter(gtffile, addchr = addchr, chrs = genome.idx, verbose = verbose)
-  para_iter = itertools.izip(gene_iter, itertools.repeat(bampath), itertools.repeat(offdict))
-  if numProc <= 1 : merge_iter = itertools.imap(_merge_cnts, para_iter)
+  para_iter = izip(gene_iter, itertools.repeat(bampath), itertools.repeat(offdict))
+  if numProc <= 1 : merge_iter = imap(_merge_cnts, para_iter)
   else : 
     pool = Pool(processes = numProc - 1)
     merge_iter = pool.imap_unordered(_merge_cnts, para_iter, chunksize = 100)
@@ -471,7 +478,7 @@ def estimate_tis_bg_all(gtfpath, bampath, genomefapath, parts = [0.25, 0.5, 0.75
     pool.close()
   return paras, slp, data
 
-def multiRibo(trans, bampaths, offdict = None, compatible = True, mis = 2, paired = False): 
+def multiRibo(trans, bampaths, offset = offset, offdict = None, compatible = True, mis = 2, paired = False): 
   '''Load multiple ribobam files to one object
   '''
   if type(bampaths) == list : 
@@ -482,7 +489,7 @@ def multiRibo(trans, bampaths, offdict = None, compatible = True, mis = 2, paire
     offdict = [offdict]
   mribo = Ribo(trans) # Empty object
   for i in range(len(bamfiles)) :
-    mribo.merge(Ribo(trans, bamfiles[i], offdict = offdict[i], compatible = compatible, mis = mis, paired = paired))
+    mribo.merge(Ribo(trans, bamfiles[i], offset = offset, offdict = offdict[i], compatible = compatible, mis = mis, paired = paired))
   return mribo
 def multiRiboGene(gene, bampaths, offdict = None, compatible = True, mis = 2, paired = False): 
   '''Load multiple ribobam files to one object
@@ -558,8 +565,8 @@ def estimate_tis_bg_inframe(genepath, bampaths, genomefapath, parts = [0.25, 0.5
   data = [exp.ReadDict() for i in range(len(parts))]
   
   gene_iter = gtf.gtfgene_iter(gtffile, addchr = addchr, chrs = genome.idx, verbose = verbose)
-  para_iter = itertools.izip(gene_iter, itertools.repeat(bampaths), itertools.repeat(offdict))
-  if numProc <= 1 : merge_iter = itertools.imap(_merge_cnts_cds, para_iter)
+  para_iter = izip(gene_iter, itertools.repeat(bampaths), itertools.repeat(offdict))
+  if numProc <= 1 : merge_iter = imap(_merge_cnts_cds, para_iter)
   else : 
     pool = Pool(processes = numProc - 1)
     merge_iter = pool.imap_unordered(_merge_cnts_cds, para_iter, chunksize = 20)
@@ -614,10 +621,11 @@ def _cdsCounts(args): # updated for bampath list
   cds2 = t.cds_stop(cdna = True)
   if cds2 is None or cds2 == cds1 : return None
   if cds2 < cds1 or (cds2 - cds1) % 3 > 0 : 
-    print('Wrong CDS: {} {} {}'.format(t.id, cds1, cds2))
+    print('Wrong CDS annotation: {} {} {}'.format(t.id, cds1, cds2))
     return None
   tis = multiRibo(t, bampaths, offdict = offdict, compatible = False, paired = paired) # not compatible
   score = tis.abdscore()
+  if score is None: return None
   genome = fa.Fa(genomefapath)
   tsq = genome.transSeq(t) #tools.trans2seq(genome, t)
   tdata = exp.ReadDict()
@@ -639,8 +647,8 @@ def estimateTISbg(genepath, bampaths, genomefapath, parts = [0.25, 0.5, 0.75], o
   
   trans_iter = io.transSelectIter(genepath, fileType = geneformat, chrs = genome.idx, verbose = verbose)
   args2 = bampaths, offdict, genomefapath, harrwidth, skip_tis, alt_tis, paired
-  para_iter = itertools.izip(trans_iter, itertools.repeat(args2)) # , itertools.repeat(offdict))
-  if numProc <= 1 : merge_iter = itertools.imap(_cdsCounts, para_iter)
+  para_iter = izip(trans_iter, itertools.repeat(args2)) # , itertools.repeat(offdict))
+  if numProc <= 1 : merge_iter = imap(_cdsCounts, para_iter)
   else : 
     pool = Pool(processes = numProc - 1)
     merge_iter = pool.imap_unordered(_cdsCounts, para_iter, chunksize = 5)
@@ -652,6 +660,9 @@ def estimateTISbg(genepath, bampaths, genomefapath, parts = [0.25, 0.5, 0.75], o
     ctotal += total
     fulldata[t.id] = tdata, score #, mcds1, mcds2
     genes.append(t)
+  if len(sl) < len(parts):
+    print('Too few TIS background samples! Known CDS annotation should be provided. See `-a` option')
+    exit(1)
   sl.sort()
   if verbose : print ('Group data...')
   slp = [None] * l
@@ -709,8 +720,8 @@ def estimate_tis_bg(gtfpath, bampath, genomefapath, parts = [0.25, 0.5, 0.75], o
   data = [exp.readdict() for i in range(len(parts))]
   
   gene_iter = gtf.gtfgene_iter(gtffile, addchr = addchr, chrs = genome.idx, verbose = verbose)
-  para_iter = itertools.izip(gene_iter, itertools.repeat(bampath), itertools.repeat(offdict))
-  if numProc <= 1 : merge_iter = itertools.imap(_max_cnts, para_iter)
+  para_iter = izip(gene_iter, itertools.repeat(bampath), itertools.repeat(offdict))
+  if numProc <= 1 : merge_iter = imap(_max_cnts, para_iter)
   else : 
     pool = Pool(processes = numProc - 1)
     merge_iter = pool.imap_unordered(_max_cnts, para_iter, chunksize = 20)
@@ -766,10 +777,12 @@ def frame_bias(arr): # ['0', '1', '2', '01', '02', '12', '012']
 class lenDis:
   '''Calculate several distributions for quality control
   '''
-  def __init__(self, lens, dis, tl = 0, cds1 = 0, cds2 = 0):
+  def __init__(self, lens, dis, tl = 0, cds1 = 0, cds2 = 0, offset = defOffset, flank = 0):
     d = dis[1] - dis[0]
     self.dis = dis
     self.lens = lens
+    self.tl, self.flank = tl, flank
+    self.tl2 = tl + flank * 2
     self.cds1, self.cds2 = cds1, cds2
     self.d1, self.d2 = {}, {} #sum of reads near start or stop condons
     self.d1d, self.d2d = {}, {} # distributions of d1, d2
@@ -777,18 +790,21 @@ class lenDis:
     self.dc = {}
     self.l = {}
     self.cnts = {}
+    self.offset = offset
     for l in range(lens[0], lens[1]):
       self.l[l] = 0
       self.d1[l] = [0] * d
       self.d2[l] = [0] * d
       self.d1d[l] = [exp.ReadDict() for i in range(d)]
       self.d2d[l] = [exp.ReadDict() for i in range(d)]
-      self.cnts[l] = [0] * tl # for transcript level
+      self.cnts[l] = [0] * self.tl2 # for transcript level, adding flanking positions
+      self.df[l] = [exp.ReadDict() for i in range(codonSize)]
   def record(self, l, i, n = 1):
     '''record a read with length l and 5' end position i
     '''
     self.l[l] += n
-    self.cnts[l][i] += n
+    i2 = i + self.flank
+    if 0 <= i2 < self.tl2: self.cnts[l][i2] += n
     ir = i - self.cds1
     if self.dis[0] <= ir < self.dis[1] : self.d1[l][ir - self.dis[0]] += n
     ir = i - self.cds2
@@ -799,7 +815,7 @@ class lenDis:
     for l in self.d1: 
       self.df[l] = [exp.ReadDict() for i in range(codonSize)] # reset
       for i in range(self.cds1, self.cds2 + codonSize, codonSize):
-        io = i - defOffset - codonSize # -15 -> stop
+        io = i - self.offset - codonSize + self.flank # -15 -> stop
         if io < 0 : continue
         for i2 in range(codonSize):
           self.df[l][i2].record(self.cnts[l][io+i2]) #
@@ -820,7 +836,7 @@ class lenDis:
       cfcnts = [[0] * length for j in range(codonSize)] # profile in 3 frames
       for si in range(length) :
         for j in range(codonSize) : 
-          io = si * codonSize + j + self.cds1 - defOffset - codonSize
+          io = si * codonSize + j + self.cds1 - self.offset - codonSize + self.flank
           if io < 0 : continue
           cfcnts[j][si] = self.cnts[l][io]
       for i in range(bins):
@@ -835,6 +851,7 @@ class lenDis:
             else : self.dc[l][j][i] += cfcnts[j][si1]
             for si in range(si1 + 1, si2) : self.dc[l][j][i] += cfcnts[j][si]
             if i < bins - 1 and si2 < stop : self.dc[l][j][i] += cfcnts[j][si2] * (stop - si2)
+            if i == bins - 1 and si2 < length : self.dc[l][j][i] += cfcnts[j][si2]
             self.dc[l][j][i] /= binsize
     return self.dc
   def merge(self, other):
@@ -914,17 +931,19 @@ def _lendis_trans(args):
   '''quality profile in each transcript
   '''
   t, bampath, lens, dis, ccds, minR, m0, cdsBins, paired = args
+  #print(t)
   bamfile = bam.Bamfile(bampath, "rb")
   tl = t.cdna_length()
   cds1, cds2 = t.cds_start(cdna = True), t.cds_stop(cdna = True) - codonSize
-  td = lenDis(lens, dis, tl, cds1, cds2)
-  if m0: tdm = lenDis(lens, dis, tl, cds1, cds2)
+  flank = - dis[0] # in case transcript annotation has no 5'UTR (starts from CDS start)
+  td = lenDis(lens, dis, tl, cds1, cds2, flank=flank)
+  if m0: tdm = lenDis(lens, dis, tl, cds1, cds2, flank=flank)
   tr = 0 # Total reads
-  for r in bam.transReadsIter(bamfile, t, compatible = False, maxNH = maxNH, minMapQ = minMapQ, secondary = secondary, paired = paired):
+  for r in bam.transReadsIter(bamfile, t, compatible=False, maxNH=maxNH, minMapQ=minMapQ, secondary=secondary, paired=paired, flank=flank):
     if m0 : ism0 = r.is_m0()
     l = r.fragment_length()
     if l < lens[0] or l >= lens[1]: continue # not in given length range
-    i = t.cdna_pos(r.genome_pos(0)) # 5' end 
+    i = t.cdna_pos(r.genome_pos(0), flank=flank) # 5' end 
     if i is None : continue
     tr += 1
     if not m0 or not ism0 : 
@@ -952,8 +971,8 @@ def lendis(genepath, bampath, lens = [25,35], dis = [-40,20], ccds = False, minR
   #gene_iter = gtf.gtfgene_iter(gtffile, addchr = addchr, verbose = verbose)
   trans_iter = io.transSelectIter(genepath, fileType = geneformat, verbose = verbose)
   rep = itertools.repeat
-  para_iter = itertools.izip(trans_iter, rep(bampath), rep(lens), rep(dis), rep(ccds), rep(minR), rep(m0), rep(cdsBins), rep(paired))
-  if numProc <= 1 : len_iter = itertools.imap(_lendis_trans, para_iter) #_lendis_gene
+  para_iter = izip(trans_iter, rep(bampath), rep(lens), rep(dis), rep(ccds), rep(minR), rep(m0), rep(cdsBins), rep(paired))
+  if numProc <= 1 : len_iter = imap(_lendis_trans, para_iter) #_lendis_gene
   else : 
     pool = Pool(processes = numProc - 1)
     len_iter = pool.imap_unordered(_lendis_trans, para_iter, chunksize = 20) #_lendis_gene
@@ -962,13 +981,13 @@ def lendis(genepath, bampath, lens = [25,35], dis = [-40,20], ccds = False, minR
     if result is None : continue
     ad.merge(result[0])
     if m0 : adm.merge(result[1])
-  pool.close()
+  if numProc > 1 : pool.close()
   results = [ad]
   if m0 : results += [adm]
   return results
 
 
-def lendisM0(gtfpath, bampath, lens = [26,35], dis = [-40,20], maxNH = maxNH, minMapQ = minMapQ, minR = 1):
+def lendisM0(gtfpath, bampath, lens = [26,35], dis = [-40,20], minR = 1):
   '''old version, do not seperate mismatch at 0
   '''
   bamfile = bam.Bamfile(bampath, "rb")
@@ -1120,9 +1139,67 @@ def formatdict(d, tab=1):
   '''show dict in ordered manner
   '''
   s = '{'
-  ks = sorted(d)
+  ks = sorted([k for k in d if type(k) == int]) + sorted([k for k in d if type(k) == str])
   for k in ks:
     if type(d[k]) == dict : s += '{}: {}, '.format(repr(k), formatdict(d[k], tab+1))
     else : s += '{}: {}, '.format(repr(k), repr(d[k]))
-  s = s[:-2] + '}'
+  s = s.strip(', ') + '}'
   return s
+
+def TIStest_betaBinom(t1, t2, r1, r2, scale_t = 1, scale_r = 1, alt = 'two.tailed', prior = [1,1]):
+  ''' Differental TIS test using Beta-Binomial model.
+      t1 & t2 are TIS raw counts. r1 & r2 are RNASeq raw counts.
+      scale_t & scale_r are scale factors of 2/1
+  '''
+  st2 = scale_t ** 0.5
+  sr2 = scale_r ** 0.5
+  alpha = (prior[0] + r1 * sr2) / st2
+  beta = (prior[1] + r2 / sr2) * st2
+  bb = stat.betaBinom(alpha, beta)
+  #if alpha == beta == 0.0 : return 1
+  return bb.pvalue(t1+t2, t1, alt = alt)
+def TIStest_chi2(t1, t2, r1, r2, scale_t = 1, scale_r = 1, alt = 'two.tailed'):
+  ''' Differental TIS test using Chi square test
+      Parameters are same as TIStest_betaBinom
+  '''
+  st2, sr2 = scale_t ** 0.5, scale_r ** 0.5
+  nt1, nt2 = t1 * st2, t2 / st2
+  nr1, nr2 = r1 * sr2, r2 / sr2
+  total = nt1 + nt2 + nr1 + nr2
+  sum1 = nt1 + nr1 #, nt2 + nr2
+  sumt = nt1 + nt2 #, nr1 + nr2
+  p1, pt = sum1 / total, sumt / total
+  p2, pr = 1 - p1, 1 - pt #sumt / total, sum2 / total
+  obs = nt1, nt2, nr1, nr2
+  exp = total * p1 * pt, total * p2 * pt, total * p1 * pr, total * p2 * pr
+  chi2, pv = stat.chisquare(obs, exp, ddof=1)
+  if math.isnan(pv) : 
+    #print('nan for {} {} {} {}!'.format(t1, t2, r1, r2))
+    return 1.0
+  if alt == 'two.sided' : return pv
+  if obs[0] >= exp[0] and alt in ('g', 'greater'): return pv / 2
+  if obs[0] <= exp[0] and alt in ('l', 'less'): return pv / 2
+  return 1 - pv / 2
+def TIStest_FisherExact(t1, t2, r1, r2, scale_t = 1, scale_r = 1, alt = 'two.tailed') :
+  ''' Differental TIS test using Fisher's exact test of rounded normalized counts
+  '''
+  st2, sr2 = scale_t ** 0.5, scale_r ** 0.5
+  if t1 + t2 > r1 + r2 :
+    nt1, nt2 = int(round(t1 * st2 / sr2)), int(round(t2 / st2 * sr2))
+    nr1, nr2 = r1, r2
+  else :
+    nt1, nt2 = t1, t2
+    nr1, nr2 = int(round(r1 * sr2 / st2)), int(round(r2 / sr2 * st2))
+  total = nt1 + nt2 + nr1 + nr2
+  sum1, sum2 = nt1 + nr1, nt2 + nr2
+  sumt, sumr = nt1 + nt2, nr1 + nr2
+  #p1, pt = sum1 / total, sumt / total
+  #p2, pr = 1 - p1, 1 - pt 
+  #exp = total * p1 * pt
+  pv =  stat.hypergeo_test(total, sum1, sumt, nt1, alt)
+  return pv
+  #if alt == 'two.sided' : return pv
+  #if nt1 >= exp and alt in ('g', 'greater'): return pv
+  #if nt1 <= exp and alt in ('l', 'less'): return pv
+  #return 1 - pv / 2
+
