@@ -4,10 +4,15 @@ Copyright (c) 2016 Peng Zhang <zhpn1024@163.com>
 '''
 
 import math
-from scipy.stats import nbinom, chisquare, chisqprob
+from scipy.stats import nbinom, chisquare # chisqprob
+from scipy.stats import chi2
+from scipy.special import betaln, betainc, gammaln
 logarr = [None] # log(N)
 logsumarr = [0] # log(N!)
-def logarr_ext(n, logarr = logarr, logsumarr = logsumarr): # prepare log values
+def logsum(n): # log(N!)
+  return gammaln(n+1)
+def logarr_ext(n) : #, logarr = logarr, logsumarr = logsumarr): # prepare log values
+  global logarr, logsumarr
   l = len(logarr)
   if l < n + 1 : 
     logarr += [None] * (n + 1 - l)
@@ -17,6 +22,9 @@ def logarr_ext(n, logarr = logarr, logsumarr = logsumarr): # prepare log values
       #print i, logarr[i]
       logsumarr[i] = logsumarr[i-1] + logarr[i]
   return logarr, logsumarr
+def betainc1(a, b, x):
+  if a == 0: return 1
+  return betainc(a, b, x)
 def data_count(data):
   total, cnt = 0, 0
   for k in data:
@@ -45,16 +53,17 @@ def fisher_method(ps):
     if p == 0 : return 0.0, -1 ###
     fs += - 2 * math.log(p)
     n += 1
-  fp = chisqprob(fs, 2 * n)
+  fp = chi2.sf(fs, 2 * n) # chisqprob(fs, 2 * n)
   return fp, fs
 
 def combination_log(n, k, show = False): 
   '''N choose K, log combination number, NATURAL LOG
   '''
+  n, k = int(n), int(k)
   if n < 0 : return None
   if k > n or k < 0: return None #None is log(0)
-  logarr_ext(n)
-  lpr = logsumarr[n] - logsumarr[k] - logsumarr[n-k]
+  #logarr_ext(n)
+  lpr = logsum(n) - logsum(k) - logsum(n-k) # logsumarr[n] - logsumarr[k] - logsumarr[n-k]
   return lpr
 
 def ACprob(x, y, r = 1): 
@@ -101,11 +110,29 @@ def hypergeo_log(N, K, n, k):
   return combination_log(K, k) + combination_log(N-K, n-k) - combination_log(N, n)
 
 def hypergeo(N, K, n, k):
+  if k < 0 or k < K+n-N: return 0
+  if k > n or k > K: return 0
   lp = hypergeo_log(N, K, n, k)
   if lp is None : return 0
   return math.exp(lp)
   
-def binomial(k, n, p = 0.5, show=False):
+def hypergeo_test(N, K, n, k, alt = 'two.tailed') :
+  p = hypergeo(N, K, n, k)
+  if alt in ('g', 'greater') :
+    for i in range(k, min(K, n)) :
+      p += hypergeo(N, K, n, i+1)
+      if p > 1 : return 1
+    return p
+  elif alt in ('l', 'less') :
+    for i in range(max(0, n + K - N), k) :
+      p += hypergeo(N, K, n, i)
+      if p > 1 : return 1
+    return p
+  p *= 2
+  if p > 1 : p = 1
+  return p
+
+def binomial(n, k, p = 0.5, show=False):
   if k > n or k < 0: return 0
   if p < 0 : p = 0
   if p > 1 : p = 1
@@ -131,7 +158,7 @@ def binomial(k, n, p = 0.5, show=False):
     pr *= pi
     if show: print (pr)
   return pr
-def binom_log(k, n, p = 0.5, show = False): #log probability value
+def binom_log(n, k, p = 0.5, show = False): #log probability value
   if n < 0 : return None
   if k > n or k < 0: return None #None is log(0)
   if p <= 0 :
@@ -144,11 +171,11 @@ def binom_log(k, n, p = 0.5, show = False): #log probability value
   lpr = math.log(p) * k + math.log(1-p) * (n-k)
   lpr += combination_log(n, k)
   return lpr
-def binom_test(k, n, p = 0.5, alt = "g", log = True, show=False): 
+def binom_test0(n, k, p = 0.5, alt = "g", log = True, show=False): 
   '''binomial test, no two sided yet!
   '''
-  if not log : return binomTest0(k, n, p, alt, show) # if log, p are calculated with log values
-  lpk = binom_log(k, n, p)
+  if not log : return binomTest0(n, k, p, alt, show) # if log, p are calculated with log values
+  lpk = binom_log(n, k, p)
   if show : print (lpk)
   if lpk is None : 
     if alt[0] == 'g' and p >= 1: return 1
@@ -161,17 +188,29 @@ def binom_test(k, n, p = 0.5, alt = "g", log = True, show=False):
   lq = math.log(q)
   if alt[0] == 'g':
     for i in range(k, n):
-      lpk += lp + logarr[n - i] - lq - logarr[i + 1] #r = p * (n - i) / q / (i + 1)
+      lpk += lp + math.log(n - i) - lq - math.log(i + 1) #r = p * (n - i) / q / (i + 1)
       pv += math.exp(lpk)
   else:
     for i in range(k, 0, -1):
-      lpk += lq + logarr[i] - lp - logarr[n - i + 1] #r = q * i / p / (n - i + 1)
+      lpk += lq + math.log(i) - lp - math.log(n - i + 1) #r = q * i / p / (n - i + 1)
       pv += math.exp(lpk)
   return pv
-def binomTest0(k, n, p = 0.5, alt = "g", show=False): # No two sided yet!
+def binom_test(n, k, p = 0.5, alt = "g"):
+  if alt[0] == 'l':
+    return betainc1(n-k, k+1, 1-p)
+  elif alt[0] == 'g':
+    return betainc1(k, n-k+1, p)
+  else:
+    if k <= n * p:
+      pv = betainc1(n-k, k+1, 1-p)
+    else:
+      pv = betainc1(k, n-k+1, p)
+    return min(pv*2, 1)
+
+def binomTest0(n, k, p = 0.5, alt = "g", show=False): # No two sided yet!
   '''binomial test no log version
   '''
-  pk = binomial(k, n, p)
+  pk = binomial(n, k, p)
   if show : print (pk)
   if pk == 1 : return 1
   if pk == 0 : 
@@ -219,9 +258,12 @@ class NegBinom:
     return nbinom.cdf(k, self.r, self.p)
   def pvalue(self, k = 0, record = True):
     if record : 
-      key = (self.p, self.q)
+      key = (self.r, self.p)
       if key not in self.p_record : self.p_record[key] = {}
       elif k in self.p_record[key] : return self.p_record[key][k]
+    if k == 0: p = 1
+    else: p = betainc(k, self.r, self.q)
+    '''
     p = 1 - nbinom.cdf(k-1, self.r, self.p)
     if p > self.Delta : 
       if record : self.p_record[key][k] = p
@@ -238,7 +280,7 @@ class NegBinom:
       ka += 1
       pa = nbinom.pmf(ka, self.r, self.p)
       p += pa
-    #p += pa
+    #p += pa'''
     if record : self.p_record[key][k] = p
     return p
       
@@ -547,14 +589,14 @@ class Poisson:
     return lpr
   def pmf(self, k = 0):
     return math.exp(self.logpmf(k))
-  def cdf(self, k = 0, logarr = logarr):
+  def cdf(self, k = 0): #, logarr = logarr):
     if k < 0 : return 0
-    logarr_ext(k, logarr = logarr)
+    #logarr_ext(k)
     lpr = self.logpmf(0)
     logl = math.log(self.l)
     cdf = math.exp(lpr)
     for i in range(1, k + 1):
-      lpr += logl - logarr[i]
+      lpr += logl - math.log(i) # logarr[i]
       cdf += math.exp(lpr)
     return cdf
   def pvalue(self, k = 0):
@@ -609,14 +651,14 @@ class ZTPoisson(Poisson):
     return lp - math.log(1 - p0)
   #def pmf(self, k = 0):
     #return math.exp(self.logpmf(k))
-  def cdf(self, k = 1, logarr = logarr):
+  def cdf(self, k = 1): #, logarr = logarr):
     if k <= 0 : return 0
-    logarr_ext(k, logarr = logarr)
+    #logarr_ext(k)
     lpr = self.logpmf(1)
     logl = math.log(self.l)
     cdf = math.exp(lpr)
     for i in range(2, k + 1):
-      lpr += logl - logarr[i]
+      lpr += logl - math.log(i) # logarr[i]
       cdf += math.exp(lpr)
     return cdf
   #def pvalue(self, k = 0):
@@ -709,8 +751,8 @@ class rankSumTiesExact:
   def complexity(self):
     '''sum of log(n+1) for each ties
     '''
-    logarr_ext(max(self.cd.values()) + 1)
-    complog = sum([logarr[self.count(i)+1] for i in range(self.l)])
+    #logarr_ext(max(self.cd.values()) + 1)
+    complog = sum([math.log(self.count(i)+1) for i in range(self.l)])
     return complog
   def shuffleTest(self, n = 1000, show = False):
     import random
@@ -731,7 +773,7 @@ class rankSumTiesExact:
   def rank(self, i):
     return self.rd[self.ks[i]]
   def numStats(self):
-    logarr_ext(max(self.cd.values()) + 1)
+    #logarr_ext(max(self.cd.values()) + 1)
     return self._numStats(self.N, self.n, 0)
   def _numStats(self, N, n, i): # NOT CORRECT!!
     n1 = N - n
@@ -741,7 +783,7 @@ class rankSumTiesExact:
     if max(self.vs[i:]) == 1 : return math.exp(combination_log(N,n))
     t1, t2 = self.count(i), N - self.count(i)
     if t1 >= n >= t2 or t1 >= n1 >= t2 : 
-      d = math.exp(sum([logarr[self.count(k)+1] for k in range(i+1, self.l)]))
+      d = math.exp(sum([math.log(self.count(k)+1) for k in range(i+1, self.l)]))
       return d
       #print 'multi all', d, N, n, i
     s = 0
@@ -760,13 +802,13 @@ class rankSumTiesExact:
     '''
     s = sum([v**3 - v for v in self.cd.values() if v > 1])
     return s / float(self.N ** 3 - self.N)
-  def test(self, th = 20, delta = 1e-4):
+  def test(self, th = 10, delta = 1e-4):
     '''if either size <= th, use fastTest
     elif complexity <= th, use fastTest
     else use normal mwtest
     '''
     if self.n <= th or self.N - self.n <= th : return self.fastTest(delta = delta) #, True
-    if self.complexity() <= th * logarr[2] : return self.fastTest(delta = delta) #, True
+    if self.complexity() <= th: return self.fastTest(delta = delta) # * logarr[2]
     return self.mwtest() #, False
   def mwtest(self, use_continuity = True, show = False):
     '''normal mwtest, alt = greater, one-tailed
@@ -964,3 +1006,63 @@ def glmNBTest(x, y):
   p = model.pvalues[1] / 2
   if model.tvalues[1] >= 0 : return p
   else : return 1-p
+
+class betaBinom:
+  ''' Beta-Binomial distribution model
+  '''
+  def __init__(self, a, b): # alpha & beta
+    self.a = a
+    self.b = b
+  def logpmf(self, n, k):
+    return combination_log(n, k) + betaln(k + self.a, n - k + self.b) - betaln(self.a, self.b)
+  def pmf(self, n, k):
+    return math.exp(self.logpmf(n, k))
+  def cdf(self, n, k):
+    p = 0
+    for i in range(k+1):
+      p += self.pmf(n, i)
+    if p > 1 : p = 1
+    return p
+  def pvalue(self, n, k, alt = 'two.sided'):
+    cdf = self.cdf(n, k)
+    if alt in ('l', 'less') : return cdf
+    else :
+      p = 1 - cdf + self.pmf(n, k)
+      if alt in ('g', 'greater') : return p
+      else :
+        m = min(cdf, p) * 2
+        if m > 1 : m = 1
+        return m
+    #if alt not in ('g', 'greater') : return cdf
+    #else :
+      #p = self.pmf(n, k)
+      #return 1 - cdf + p
+
+def hw_exact_test(n, m, k, alt = 'two.tailed'): #pass
+  p = 0
+  for i in range(m+1):
+    p += hypergeo(n*2, n, m, i) * hypergeo_test(n, i, m-i, k, alt=alt)
+  return p
+
+def HWtest(AA, Aa, aa):
+  n = float(AA + Aa + aa)
+  if n == 0: return 1, 0
+  pA = (2*AA + Aa)/(2*n)
+  pa = 1 - pA # (Aa + 2*aa)/(2*n)
+  EAA = n*(pA**2)
+  EAa = n*(2*pA*pa)
+  Eaa = n*(pa**2)
+  #if aa > Eaa: return 1 # only test excess het
+  if (EAA>=5) and (EAa>=5) and (Eaa>=5) and n>=40:
+    chisqr = ((AA-EAA)**2)/EAA + ((Aa-EAa)**2)/EAa + ((aa-Eaa)**2)/Eaa
+    p = chi2.sf(chisqr, 1)
+  else:
+    m = 2*aa + Aa
+    n = int(n)
+    #if aa > Eaa: p = 2 * chunhe_test(n, m, aa, alt='g')
+    if aa > Eaa: p = 2 * hw_exact_test(n, m, aa, alt='g')
+    else: p = 2 * hw_exact_test(n, m, aa, alt='l')
+    #p = 2 * hw_exact_test(n, m, aa, alt='l')
+    if p > 1: p = 1
+  return p, EAa
+
